// --- [ TWI_Master_Slave.h ] -------------------------------------------------
//       10.11.08		tab = 3
//       Header file fr TWI_Master_Slave.c
//
// ----------------------------------------------------------------------------

#define TWI_CLOCK      	102400			// 100kHz
#define TWI_TWBR 	((F_CPU / (2 * TWI_CLOCK)) - 8)
// TWBR bei FCPU 3686400 fr: 100KHz = 10, 50KHz = 29, 25KHz = 64, 12 KHz = 145
// #define TWI_TWBR         0x0A       // TWI Bit rate Register setting.
                                       // Set Application note for detailed
                                       // information on setting this value.

union TWI_statusReg                    // Status byte holding flags.
{
   uint8_t all;
   struct
   {
      uint8_t LastTransOK:1;
      uint8_t RxDataInBuf:1;
      uint8_t GenAddressCall:1;  		// TRUE = General call, FALSE = TWI Address;
      uint8_t unusedBits:5;
   };
};

// TWI_Mode:
#define  PASSIV	   0
#define  SLAVE		   1
#define  MASTER	   2
#define  ACTIV   	   4

// Slave_Mode - zeigt die letzte Slave Aktion an: Transmit oder Receive
// bei einem Transmit liefert TWI_SLA_Received() immer False zurck
#define  TRANSMIT	   0
#define	RECEIVE	   1

void 		TWI_Initialise( void );
uint8_t 	TWI_Transceiver_Busy( void );
void     TWI_Set_Repeated_Start(void);
uint8_t 	TWI_Get_State_Info( void );
uint8_t  TWI_Get_Err(void);
void 		TWI_SLA_Start_Transceiver( void );
uint8_t  TWI_SLA_Received(void);
void 		TWI_MA_Transmit_Data( uint8_t, uint8_t, uint8_t * );
void 		TWI_MA_Start_Transceiver( void );

// anstelle von public-Variablen TWI_statusReg erfolgt die Abfrage ber Funktionen
// uint8_t 	TWI_LastTransOK(void);


// ---  TWI State codes -------------------------------------------------------

// General TWI Master status codes
#define TWI_START                  0x08  // START has been transmitted
#define TWI_REP_START              0x10  // Repeated START has been transmitted
#define TWI_ARB_LOST               0x38  // Arbitration lost

// TWI Master Transmitter status codes
#define TWI_MTX_ADR_ACK            0x18  // SLA+W has been transmitted and ACK received
#define TWI_MTX_ADR_NACK           0x20  // SLA+W has been transmitted and NACK received
#define TWI_MTX_DATA_ACK           0x28  // Data byte has been transmitted and ACK received
#define TWI_MTX_DATA_NACK          0x30  // Data byte has been transmitted and NACK received

// TWI Master Receiver status codes
#define TWI_MRX_ADR_ACK            0x40  // SLA+R has been transmitted and ACK received
#define TWI_MRX_ADR_NACK           0x48  // SLA+R has been transmitted and NACK received
#define TWI_MRX_DATA_ACK           0x50  // Data byte has been received and ACK transmitted
#define TWI_MRX_DATA_NACK          0x58  // Data byte has been received and NACK transmitted

// TWI Slave Transmitter status codes
#define TWI_STX_ADR_ACK            0xA8  // Own SLA+R has been received; ACK has been returned
#define TWI_STX_ADR_ACK_M_ARB_LOST 0xB0  // Arbitration lost in SLA+R/W as Master;
													  // own SLA+R has been received; ACK has been returned
#define TWI_STX_DATA_ACK           0xB8  // Data byte in TWDR has been transmitted;
                                         // ACK has been received
#define TWI_STX_DATA_NACK          0xC0  // Data byte in TWDR has been transmitted;
                                         // NOT ACK has been received
#define TWI_STX_DATA_ACK_LAST_BYTE 0xC8  // Last data byte in TWDR has been transmitted (TWEA = 0);
                                         // ACK has been received

// TWI Slave Receiver status codes
#define TWI_SRX_ADR_ACK            0x60  // Own SLA+W has been received ACK has been returned
#define TWI_SRX_ADR_ACK_M_ARB_LOST 0x68  // Arbitration lost in SLA+R/W as Master; own SLA+W has been received;
													  // ACK has been returned
#define TWI_SRX_GEN_ACK            0x70  // General call address has been received; ACK has been returned
#define TWI_SRX_GEN_ACK_M_ARB_LOST 0x78  // Arbitration lost in SLA+R/W as Master; General call address has been received;
													  // ACK has been returned
#define TWI_SRX_ADR_DATA_ACK       0x80  // Previously addressed with own SLA+W; data has been received;
                                         // ACK has been returned
#define TWI_SRX_ADR_DATA_NACK      0x88  // Previously addressed with own SLA+W; data has been received;
                                         // NOT ACK has been returned
#define TWI_SRX_GEN_DATA_ACK       0x90  // Previously addressed with general call; data has been received;
                                         // ACK has been returned
#define TWI_SRX_GEN_DATA_NACK      0x98  // Previously addressed with general call; data has been received;
                                         // NOT ACK has been returned
#define TWI_SRX_STOP_RESTART       0xA0  // A STOP condition or repeated START condition has been received
                                         // while still addressed as Slave

// TWI Miscellaneous status codes
#define TWI_NO_STATE               0xF8  // No relevant state information available; TWINT = 0
#define TWI_BUS_ERROR              0x00  // Bus error due to an illegal START or STOP condition

// --- [ eof ] ----------------------------------------------------------------
