// --- [ global.h ] ----------------------------------------------------------
//       12.07.2013		tab = 3
//       zu pc_2_twi.c
// ----------------------------------------------------------------------------

#ifndef  EXTERN
#define EXTERN extern
#endif

#define TRUE               1
#define FALSE              0

EXTERN uint8_t rs_buf[256];
EXTERN uint8_t ma_buf[256];
EXTERN uint8_t sla_buf[256];
EXTERN uint8_t msg[24];

// diese Variablen werden in Interruptroutinen genutzt
//EXTERN volatile uint8_t rs_head;
#define rs_head         GPIOR1
EXTERN volatile uint8_t sla_head, sla_tail;
EXTERN volatile uint8_t int_tail, int_head;
//EXTERN volatile uint8_t int_flag;
#define int_flag        GPIOR0
EXTERN volatile uint8_t timerovl;

EXTERN volatile uint8_t * int_buf;        // Pointer auf die Daten, die von der USART_ISR verschickt werden soll
EXTERN volatile uint8_t * twi_buf;

#define TWI_SLA_ADR        0x04           // die eigene Adresse als TWI-Slave,
                                          // die Daten werden via DATAMODE an den PC weitergereicht

#define STARTBYTE          0xA5           // erstes Byte einer bertragung via RS232

 // Frametyps
#define TWOWIRE            0xE1           // TWI-Kommunikation, Write ohne Antwort (auer bei Fehler)
#define ONEWIRE            0xE2           // DS18B20 - Temperatur, verzgerte Antwort | Fehlermeldung bei Pinfehler
#define ADCREAD            0xE3           // unmittelbare Antwort | Fehlermeldung bei Pinfehler
#define INTMODE            0xE4           // Zeitpunkt unvorhersebbar - Rckgabe bei Sense-Fehler - Fehlermeldung bei Pinfehler
#define LOGMODE            0xE5           // Sendezeitpunkt unvorhersehbar
#define PORTMODE           0xE6           // keine Antwort
#define PORTREAD           0xE7           // unmittelbare Antwort
#define PORTWRITE          0xE8           // keine Antwort
#define PORTTOG            0xE9           // keine Antwort
#define TWISPEED           0xEA           // Rckgabe: 0xA5, 0xEA, 0x01, TWBR-Register


// INTMODE: Sense 0x00 = LowLevel | 0x01 = AnyChange | 0x02 = FallingEdge | 0x03 = RisingEdge

// Folgende Fehler werden an den PC zurckgemeldet:
// TWI_ERR, Fehler bei der TWI-Kommunikation
// ADC_ERR, falsche Pinangabe
// PIN_ERR, falsche Pinangabe
// INT_ERR, falsche Pinangabe

#define TWI_ERR            0x5A              // (0xA5, 0x5A, 0x01, TWI_ADR)
#define OWI_ERR            0x6A              // (0xA5, 0x6A, 0x01, Pin_Maske)
#define ADC_ERR            0x7A              // (0xA5, 0x7A, 0x01, Pin_Nr)
#define INT_ERR            0x8A              // (0xA5, 0x8A, 0x01, Pin_Nr)

// Pins, die genutzt werden knnen
#define PORT_B_MASK     ((1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3) | (1<<PB4) | (1<<PB5) | (0<<PB6) | (0<<PB7))
#define PORT_C_MASK     ((1<<PC0) | (1<<PC1) | (1<<PC2) | (1<<PC3) | (0<<PC4) | (0<<PC5) | (0<<PC6))
#define PORT_D_MASK     ((0<<PD0) | (0<<PD1) | (1<<PD2) | (1<<PD3) | (1<<PD4) | (1<<PD5) | (1<<PD6) | (1<<PD7))


// rs_state - der Status whrend des Empfanges
#define RS_WAIT            0              // warten auf den STARTBYTE als erstem Byte
#define RS_FRAME           1              // STARTBYTE empfangen, warten auf frametyp-Byte
#define RS_START           2              // frametyp-Byte empfangen, warten auf Lngen-Byte
#define RS_READ            4              // die Datenlnge ist bekannt, es wird gelesen
#define RS_READY           8              // alle angekndigten Bytes sind eingegangen
#define RS_CHECK_TX        16             // warten auf den Abschluss eines Write
#define RS_CHECK_RX        32             // warten auf den Abschluss eines Read
#define RS_KONVERT         64             // Konvertierungszeit abwarten bei DS18B20

// Timer0 beendet den Empfang, wenn nicht die erwartete Anzahl von Bytes eintrifft
// der Timer wird nach dem jedem Empfang eines neuen Bytes zurckgesetzt.
#define TIMER0_STOP     0
#define TIMER0_START    (0<<CS02 | 1<< CS01 | 1<< CS00)  // Prescale 64

// Timer1 definiert die Warte/Konvertierzeit fr das Auslesen der DS18B20-Sensoren
#define TIMER1_STOP     0
#define TIMER1_START    (1<<CS12 | 0<<CS11 | 1<<CS10)    // Prescale 1024
#define TCNT1_HIGH      256 - 10
#define TCNT1_LOW       0


// --- [ eof ] ----------------------------------------------------------------
