// --- [ pc_2_twi.c ] ---------------------------------------------------------
//       15.07.2013      Tab = 3
//
//		   Beschreibung ausgelagert in "rs232_2_twi.txt"
//
// ----------------------------------------------------------------------------

// todo
// TWI-Takt von aussen vernderbar machen

#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define EXTERN
#include "global.h"
#include "TWI_Master_Slave.h"
#include "rs232.h"
#include "owi_lib_sx.h"
#include "porttools.h"

// ---------------------------------------------------------------------------
// Wenn beim Empfangen von der Seriellen Schnittstelle weniger Bytes eintreffen,
// als angekndigt, dann beendet der Timer den Empfang, nachdem 4ms lang keine
// Daten mehr eingetroffen sind.
// Bei 9600 Baud bentigt die bertragung eines Bytes ca. 1ms.

ISR(TIMER0_OVF_vect)
{
   timerovl = TRUE;
}

// -----------------------------------------------------------------------------
ISR(TIMER1_OVF_vect)
{
   TCCR1B = TIMER1_STOP;
}

// -----------------------------------------------------------------------------
ISR(INT0_vect)
{
   int_flag |= 0x01;                   // Flag fr main()
   EIMSK &= ~(1<<INT0);                // Interrupt0 abschalten
}

// -----------------------------------------------------------------------------
ISR(INT1_vect)
{
   int_flag |= 0x02;                   // Flag fr main()
   EIMSK &= ~(1<<INT1);                // Interrupt1 abschalten
}

// -----------------------------------------------------------------------------
ISR(PCINT0_vect)
{
   int_flag |= 0x10;
}

// -----------------------------------------------------------------------------
ISR(PCINT1_vect)
{
   int_flag |= 0x20;
}

// -----------------------------------------------------------------------------
ISR(PCINT2_vect)
{
   int_flag |= 0x40;
}

// ----------------------------------------------------------------------------
int main(void)
{
uint8_t tail = 0, head = 0;               // Anfang / Ende des Datenbereiches
uint8_t rs_tail = 0;                      // Anfang des Empfangspuffers
uint8_t ma_tail = 0, ma_head = 0;         // Anfang / Ende des TWI_Master-Puffers
uint8_t rs_state =  RS_WAIT;              // aktueller Status einer Datenbertragung
uint8_t frametyp = 0, twi_adr = 0;
uint8_t temp, i = 0;
uint8_t rs_cnt = 0, data_len = 0;         // Zhler fr die Datenlnge
uint8_t owi_mask = 0, owi_busy = FALSE;
//uint8_t ddr, port;                        // portstate fr ADC merken
uint8_t read_len;

// globale volatile variablen
sla_head = sla_tail = 0;               // Anfang / Ende des TWI-Slave-Puffers
rs_head = 0;                           // Anfang des Empfangspuffers
int_flag = 0;                          // Flag fr den Empfang eines externen Interrupts

//DDRB  |= (1<<PB0 | 1<<PB1 | 1<<PB2) ; // Debuggen
//PORTB |= (1<<PB0 | 1<<PB1 | 1<<PB2);

rs232_init();

MCUCR = (1<<SE);								// Sleep enablen (im Idle-Modus)
PRR   = (1<<PRTIM2 | 1<<PRSPI);        // SPI unjd Timer2 disablen
ACSR  = (1<<ACD);								// Analog comparator disablen

sei();

// Timer 0 verhindert, dass unvollstndige Nachrichten das System blockieren
// wenn ber die Dauer von mehr als 4.4ms keine Daten empfangen werden,
// dann wird rs_state auf RS_WAIT gesetzt und auf den Beginn einer neuen Nachricht gewartet
//TCCR0B = TIMER0_STOP;
TIMSK0 = (1<< TOIE0);

// Timer 1 vorbereiten, er steuert lngere Wartezeiten
// TCCR1B = TIMER1_STOP;
TIMSK1 = (1<<TOIE1);

TWI_Initialise();							      // Slave mit Adresse 254 initialisieren
TWI_SLA_Start_Transceiver();        	   // und startet dann den Slave-Modus

while (1)
{
   // -------------------------------------  der Timer0 hat zugeschlagen, es sind weniger Bytes eingetroffen
   if (timerovl)                          // als im Header angekndigt.
   {                                      // zurck in den Ausgangszustand
      timerovl = FALSE;
      TCCR0B   = TIMER0_STOP;
      rs_state = RS_WAIT;
   }

  // --------------------------------------  wenn mglich, dann in den SLEEP-Modus wechseln, aber nur wenn:
   if (rs_state == RS_WAIT)               // - keine Empfangsaktivitten
   {
      if (!(TWI_Transceiver_Busy()))      // - keine TWI-Aktivitten
      {
         if (!(USART_BUSY))               // - keine Sendeaktivitten an Serieller Schnittstelle
         {
            if (owi_busy == FALSE)        // - und keine Temperaturmessung im Gange ist
            {
               if (!(TWI_SLA_Received())) // - keine Daten vom TWI-Slave empfangen wurden
               {
                  if (int_flag == 0)      // - und kein Interrupt mehr bearbeitet werden muss
                  {
//                     PORTB &= ~(1<<PB0 );    // ####

                     SMCR = (1<<SE);         // Idle Modus
                                          // warten, bis sich ein Interrupt ereignet
                     asm volatile("sleep");	// RXIE, TWIE oder INT0/1 beendet das Warten

//                     PORTB |= (1<<PB0);      // #####
                  }
               }
            }
         }
      }
   }												         // Sleep wurde beendet{

   // -------------------------------------------  es sind Daten via Serielle Schnittstelle empfangen worden
   if (!(rs_tail == rs_head))         	         // wenn Daten im Eingangspuffer
	{
      if (rs_state < RS_READY)
      {

         temp = rs_buf[rs_tail++];              // Das erstes Byte im Puffer nach temp
         if (rs_state == RS_WAIT)               // wenn das erstes Byte erwartet wird
         {
            if (temp == STARTBYTE)              // prfen, ob es der STARTBYTE ist
            {
               tail = (rs_tail + 2);            // hier beginnt der Datenbereich
               rs_state = RS_FRAME;

               TCNT0 = 0;
               TCCR0B = TIMER0_START;
            }
         // wenn das erste Byte != STARTBYTE, dann bleibt rs_state unverndert
                 //put_c(frametyp);
               //put_c(data_len);
               //for (i = tail; i < tail + data_len; i++) put_c(rs_buf[i]);
               //put_c(rs_buf[tail + 1]);
               //put_c(head);
               //put_c(data_len);
               //put_c(32);
               //head = rs_tail;                  // hier endet der Datenbereich
               //head = tail + data_len;

         }
         else if (rs_state == RS_FRAME)         // zweites Byte auf gltigen Frametyp prfen und abspeichern
         {
            if ( (temp >= TWOWIRE) && (temp <=TWISPEED) )
            {
               rs_state = RS_START;             // als nchstes wird ein Byte mit der Lngenangabe erwartet
               TCNT0 = 0;
               frametyp = temp;
            }

            else                                // wenn kein gltiger frametyp bertragen wird, dann den Empfang beenden
            {
               rs_state = RS_WAIT;
            }
         }

         else if (rs_state == RS_START)         // drittes Byte = Anzahl der erwarteten Bytes
         {
            if (temp)                           // wenn eine Anzahl angebenen, 0 ist unzulssig
            {
               TCNT0 = 0;
               rs_state = RS_READ;
               data_len = rs_cnt = temp;
               head = tail + temp;
            }

            else                                // Beenden die Anzahl == 0
            {
               TCCR0B = TIMER0_STOP;
               rs_state = RS_WAIT;
            }
         }

         else if (rs_state == RS_READ)          //  hier werden die Daten empfangen, die Anzahl steht in rs_cnt
         {
            TCNT0 = 0;
            if((--rs_cnt) == 0)                 // wenn keine weiteren Bytes erwartet werden
            {
               rs_state = RS_READY;             // alle angekndigten Bytes empfangen
               TCCR0B = TIMER0_STOP;

               //put_c(frametyp);
               //put_c(data_len);
               //for (i = tail; i < tail + data_len; i++) put_c(rs_buf[i]);
               //put_c(rs_buf[tail + 1]);
               //put_c(head);
               //put_c(data_len);
               //put_c(32);
               //head = rs_tail;                  // hier endet der Datenbereich
               //head = tail + data_len;
            }
         }
      }  // if rs_state < RS_READY
	}   // Daten wurden via Serieller Schnittstelle empfangen



   // -------------------------------------------------------------------------
   // wenn eine bertragung abgeschlossen ist, dann ist rs_state == RS_READY
   if (rs_state == RS_READY)
   {
   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, sie sollen Interrupts gendert werden
   // rs_buf[0] = Interrupt [0/1],
   // rs_buf[1] = Sense Control
   // Interrupt 0 - 0 = LowLevel, 1 = Any logic change, 2 = falling edge, 3 = rising edge
   // Interrupt 1 * wie Interrupt1, aber um 2 Bit nach links schieben
   if (frametyp == INTMODE)
   {
      if (data_len == 2) int_set(tail);
      rs_state = RS_WAIT;
   }
   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, sie sollen via TWI ausgeben werden
   // die Lnge des Datenbereiches ist variabel
   // Solange noch ein TWI-bertragung im Gange ist
   else if ( (frametyp == TWOWIRE) && (!( TWI_Transceiver_Busy() )) )
   {
       // Zu senden sind die Bytes zwischen tail und head.
       // Read und Write erfolgen interrruptgesteuert im Hintergrund,
       // in beiden Fllen wird nach Abschluss der bertragung der
       // auf TWI-Fehler geprft.
       // Bei einem Read werden zustzlich die empfangenen Daten
       // an den PC ausgeliefert.
       twi_adr = rs_buf[tail];

       if ( (twi_adr & 0x01) && (data_len < 5) ) // das Read-Bit einer TWI-Adresse ist gesetzt
       {
           // data_len == 2, wenn direkt ausgelesen werden soll (TWI-Adr + cnt)
           // wenn data_len = 3 oder 4, dann wird ein Register auf dem Slave adressiert
           // diese Adresse muss zuerst gesendet werden,
           // dabei wird mit einem Repeated Start gearbeitet.

           if ( data_len > 2 )                     // dann soll noch eine Register-Adresse
            {                                      // gesendet werden, 1 oder 2 Byte lang
               rs_buf[tail] &= 0xFE;               // das R/W-Bit lschen
               TWI_Set_Repeated_Start();           // das nachfolgende TWI-Write ohne Stop abschlieen
                                                   // und mit einem Repeated Start zum Lesen bergehen.

                                                   // Das Byte mit der Anzahl zu lesender Bytes nicht senden
               TWI_MA_Transmit_Data(tail, (tail + data_len - 1), rs_buf);
               while (TWI_Transceiver_Busy());     // warten, bis die Sendung abgeschlossen ist
            }
          // Es wird hier der Datensatz fr die Rckmeldung im Array ma_buf[] vorbereitet.
          // Der Frametyp wird in ma_buf[] abgelegt, es folgt die Lnge und die TWI-Adresse.
          // Die TWI-Adresse wird dabei direkt zum Adressierung des Slaves genutzt.
          // Die empfangenen Daten werden hinten angehngt

          // Die Anzahl der zu lesenden Bytes stehen im letzten Byte
          read_len = rs_buf[(uint8_t)(tail + data_len - 1)]; // ohne cast auf uint8_t entstehen Fehler bei berlauf
          ma_buf[ma_head++] = TWOWIRE;          // den Dienst voranstellen
          ma_buf[ma_head++] = read_len + 1;     // die Anzahl zu lesender Bytes einbauen  + 1 fr die TWI-Adresse
                                                // Hier beginnt der Datenbereich
          ma_buf[ma_head] = twi_adr;            // mit der TWI-Adresse des Absenders
          ma_tail = ma_head ;                   // ma_tail  -> die TWI-Adr muss gesendet werden
          ma_head += read_len;                  // ma_head  -> bis hier hin werden Daten eingelesen

         if (read_len)         // das Lesen macht nur Sinn, wenn Daten gelesen werden sollen
         {
            TWI_MA_Transmit_Data(ma_tail, ma_head, ma_buf);
            rs_state = RS_CHECK_RX;             // die eingehenden Daten werden weiter unten gelesen
         }

         else
         {
            rs_state = RS_WAIT;
         }
      }

      else if (!(twi_adr & 0x01))  // wenn das R/W-Bit nicht gesetzt ist: TWI-Write
      {
         // Beim Schreiben werden die Daten direkt aus dem rs_buf[] geholt.
         // Die Pointer auf Datenanfang und Datenende werden als Kopie bergeben,
         // so dass whrend des TWI-Transfers eine weitere Message seriell empfangen werden kann.
         // Ein Pointer auf den DatenBuffer wird bergeben.

          TWI_MA_Transmit_Data(tail, head, rs_buf);
          rs_state = RS_CHECK_TX;               // der Erfolg der bertragung wird weiter unten geprft
      }

      else     // zu viele Bytes bei einem Read
      {
         rs_state = RS_WAIT;
      }
   }


   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, es sollen Temperatursensoren ausgelesen werden
   // bergeben wird die Pinmaske der an PORTD angeschlossenen Sensoren.
   else if (frametyp == ONEWIRE )
   {
      // es werden genau 1 Byte bergeben, die Maske der auszulesenden Pins
      // Byte 1 = Aktion = Start_Konvert oder Daten auslesen
      if (data_len == 1)
      {
         owi_mask = rs_buf[tail] & 0b11111100;  // PD.2 bis PD.7

         // wenn unpltige Maske gewhlt, keine DQ darin enthalten
         if ((owi_mask & ONE_WIRE_PIN_MASK) == 0)
         {
            msg[0] = OWI_ERR;
            msg[1] = 1;
            msg[2] = rs_buf[tail];
            put_msg();
            //rs_state = RS_WAIT;

         }
         else
         {
            OW_convert(owi_mask);  // wird zum Auslesen der Sensoren noch bentigt
            // der Timer1 signalisiert den Ablauf der Wartezeit
            TCNT1H = TCNT1_HIGH;
            TCNT1L = TCNT1_LOW;
            TCCR1B = TIMER1_START;
            owi_busy = TRUE;
            //rs_state = RS_WAIT;   // es knnen bereits andere Aktionen ausgefhrt werden
         }
      }                             // immer (auch bei falscher Parameteranzahl) mit RS_WAIT abschlieen
      rs_state = RS_WAIT;
   }



   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, es soll der Portmodus (DDR-Register) gesetzt werden
   // bergeben wird der Port [0,1,2,3] sowie zwei Pinmasken
   else if ( frametyp == PORTMODE )
   {
      if (data_len == 3)  port_mode(tail);
      rs_state = RS_WAIT;     // immer (auch bei falscher Parameteranzahl) mit RS_WAIT abschlieen
   }

   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, es sollen Portpins ausgelesen werden
   // bergeben wird der Port [0,1,2,3]
   // Zurckgeben wird das DDR-Register sowie der Pinstatus (PORT-Register)
   else if ( frametyp == PORTREAD )
   {
      if (data_len == 1) port_read(tail);
      rs_state = RS_WAIT;  // immer (auch bei falscher Parameteranzahl) mit RS_WAIT abschlieen
   }

   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, es sollen Portpins gesetzt werden
   // bergeben wird der Port [0,1,2,3] sowie zwei Pinmasken (Pins und State)
   else if ( frametyp == PORTWRITE )
   {
      if (data_len == 3) port_write(tail);
      rs_state = RS_WAIT;     // immer (auch bei falscher Parameteranzahl) mit RS_WAIT abschlieen
   }

   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, es soll der Pinmodus gestoggelt werden
   // bergeben wird der Port [0,1,2,3] sowie eine Pinmaske der zu togglenden Pins.
   else if ( frametyp == PORTTOG )
   {
      if (data_len == 2) port_tog(tail);
      rs_state = RS_WAIT;  // immer (auch bei falscher Parameteranzahl) mit RS_WAIT abschlieen
   }


   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, es die Spannunng an PORTC gemessen werden
   // bergeben wird die Nr. des Pins, an dem die Messung vorgenommen wird [0 .. 3]
   // Die anderen Einstellung fr den ADC sind fest vorgegeben:
   // in ADMUX 0<<REFS1 | 0<<REFS0 | 0<<ADLAR
   // in ADCSRA 1<<ADPS2 | 0<<ADPS1 | 1<<ADPS0 Division Factor 32 -> ca. 100KHz Takt (Soll 50..200 KHz)
  else if ( frametyp == ADCREAD )
   {
      if (data_len == 1)
      {
         i = (rs_buf[tail]);
         if (i < 4)                                // PC.4 und PC.5 sind durch den TWI-Port belegt PC.6 = Reset
         {
//            ddr = DDRC;                            // das DDRC-Register sichern,
//            DDRC &= ~i;                            // den ADC-Pin als Eingang schalten
//            port = PORTC;                          // das PORTC-Register sichern
//            PORTC &= ~i;                           // ggf. gesetzte Pullups abschalten

            ADCSRA = (1<<ADEN | 0<<ADSC | 0<<ADATE | 0<<ADIE |1<<ADIF | 1<<ADPS2| 0<<ADPS1 | 1<<ADPS0);
            ADMUX = i;
            ADCSRA |= (1<<ADSC);                   // Messung starten

            while (!(ADCSRA & 1<<ADIF));           // auf den Abschluss warten

//            DDRC = ddr;                            // die alten Einstellungen wieder restaurieren
//            PORTC = port;

            msg[0] = ADCREAD;
            msg[1] = 3;
            msg[2] = i;
            msg[4] = ADCL;                         // zuerst muss ADCL ausgelesene werden
            msg[3] = ADCH;
            put_msg();
         }

         else                                      // Fehlermeldung
         {
            msg[0] = ADC_ERR;
            msg[1] = 1;
            msg[2] = i;
            put_msg();
         }
      }
      rs_state = RS_WAIT;                          // immer mit RS_WAIT abschlieen
   }

   // -------------------------------------------------------------------------
   // die vollstndigen Daten sind empfangen, es soll der TWI-Takt gendert werden
   // zulssig sind die Paramter [100, 50, 25 oder 12],
   // sie geben den gewnschten TWI-Takt an.
   // Zurckgegen wird der aktuelle Wert des TWBR Registers
   else if ( frametyp == TWISPEED )
   {
      if (data_len == 1)
      {
         if      (rs_buf[tail] == 100) TWBR = 10;
         else if (rs_buf[tail] ==  50) TWBR = 29;
         else if (rs_buf[tail] ==  25) TWBR = 64;
         else if (rs_buf[tail] ==  12) TWBR = 145;

         // immer den aktuellen Wert des Registers TWBR zurckgeben
         msg[0] = TWISPEED;
         msg[1] = 1;
         msg[2] = TWBR;
         put_msg();
      }
      rs_state = RS_WAIT;     // immer (auch bei falscher Parameteranzahl) mit RS_WAIT abschlieen
   }
   }                          // if rs_state = rs_ready

   // -----------------------------------------------------------------------------
   // INT0 oder INT1 setzen dieses Flag auf 0x01, 0x02, der PCINT setzt die Bit 0x10, 0x20, 0x40
   // Achtung: um Prellen zu vermeiden wird der Interrupt jeweils in der INT0/1-ISR deaktiviert
   // ggf. neu setzen
   // Die PinCange Interrupts werden nicht deaktiviert, sie mssen explizit zurckgesetzt werden !
   //
   // Das Flag (noch) nicht abarbeiten, wenn noch eine TX-bertragung im Gange ist
   if ( (int_flag) && (!(USART_BUSY) ) )
   {
      if (int_flag & 0x01)             // INT0
      {
            int_flag &= ~0x01;         // Flag lschen
            msg[0] = INTMODE;
            msg[1] = 3;                // Lnge des Datensatzes
            msg[2] = 0;                // 0 fr INT0
            msg[3] = (EICRA & 0x03);
            msg[4] = PIND & (1<<PD2);  // Pinstatus mitgeben
            put_msg();
      }

      else if (int_flag & 0x02)        // INT1
      {
            int_flag &= ~0x02;         // Flag lschen
            msg[0] = INTMODE;
            msg[1] = 3;                // Lnge des Datensatzes
            msg[2] = 1;                // 1 fr INT1
            msg[3] = (EICRA >> 2);
            msg[4] = PIND & (1<<PD3);  // Pinstatus mitgeben
            put_msg();
      }

      else if (int_flag & 0x10)        // PCINT0
      {
            int_flag &= ~0x10;         // Flag lschen
            msg[0] = INTMODE;
            msg[1] = 3;                // Lnge des Datensatzes
            msg[2] = 0x10;             // fr PCINT0
            msg[3] = PCMSK0;
            msg[4] = PINB & PCMSK0;    // Pinstatus mitgeben
            put_msg();
      }

      else if (int_flag & 0x20)        // PCINT1
      {
            int_flag &= ~0x20;         // Flag lschen
            msg[0] = INTMODE;
            msg[1] = 3;                // Lnge des Datensatzes
            msg[2] = 0x11;             // fr PCINT1
            msg[3] = PCMSK1;
            msg[4] = PINC & PCMSK1;    // Pinstatus mitgeben
            put_msg();
      }

      else if (int_flag & 0x40)       // PCINT2
      {
            int_flag &= ~0x40;         // Flag lschen
            msg[0] = INTMODE;
            msg[1] = 3;                // Lnge des Datensatzes
            msg[2] = 0x12;             // fr PCINT2
            msg[3] = PCMSK1;
            msg[4] = PIND & PCMSK2;    // Pinstatus mitgeben
            put_msg();
      }
   }

   // -------------------------------------------------------------------------
   // Erfolgskontrolle und Ausgabe der Daten nach einem TWI-Read
   if (rs_state == RS_CHECK_RX)
   {
      if ( (!( TWI_Transceiver_Busy())) && ( !(USART_BUSY)) )   // Warten bis die Antwort eingegangen ist
                                             // aber auch TX via Serielle Schnittstelle muss bereit sein
      {
         if (TWI_Get_Err() )                 // prfen, ob sich ein Fehler beim Senden ereignet hat
         {
            put_err(twi_adr);                // Wenn ja, dann Fehlermeldung zurckgeben
         }

         else                                // Wenn nein, dann die vom Slave empfangenen Daten
         {                                   // via Serielle Schnittstelle zurckliefern
            int_buf  = ma_buf;               // Pointer auf das Array mit den Daten stellen
            int_tail = ma_tail - 2;          // Anfang  der Daten bergeben
            int_head = ++ma_head;            // Ende des Datenbereiches

            UCSR0B = TX_ENABLE;              // Interrupt enablen
            UDR0 = STARTBYTE;                // Das STARTBYTE als erstes Byte auf die Reise schicken
                                             // den Rest erledigt die ISR
         }
         rs_state = RS_WAIT;                 // bereit zu neuen Taten
      }
   }

   // -------------------------------------------------------------------------
   // Erfolgskontrolle nach dem TWI-Write
   if (rs_state == RS_CHECK_TX)
   {
         if ( !( TWI_Transceiver_Busy()) )   // wenn der Master fertig ist
         {
                                             // prfen, ob sich ein Fehler beim Senden ereignet hat
            if (TWI_Get_Err()) put_err(twi_adr);
                                             // bei Erfolg keine Rckmeldung senden
            rs_state = RS_WAIT;
         }
   }

   // -------------------------------------------------------------------------
   // Temperaturmessung mit Dallas DS18B20
   // nach dem Ablauf der Konvertierungszeit knnen die Werte ausgelesen werden
   if (owi_busy == TRUE)
   {
      if ((TCCR1B == TIMER1_STOP) && (!(USART_BUSY))) // wenn sich der Timer abgeschaltet hat
      {
         owi_busy = FALSE;
         msg[0] = ONEWIRE;
         OW_readall(&msg[1], owi_mask);
         put_msg();                          // die Daten senden
      }
   }

   // -------------------------------------------------------------------------
	// Wenn der TWI-Slave Daten empfangen hat
	if (TWI_SLA_Received())   				   // wenn der Slave Daten vollstndig empfangen hat
	{
      // Lnge des Datenbereiches berechnen unter Bercksichtigung des berlaufes
      if (sla_head > sla_tail) temp = sla_head - sla_tail;
      else temp = (sla_head + (255 - sla_tail));

      if (temp > 1)                       // bei einem Ping wird nur 1Byte bergeben = eigene Adresse
      {
         // Datensatzlnge und Frametyp voranstellen
         // das schrnkt die Anzahl der Daten auf 252 ein - sollte praktisch kein Problem sein !
         // Das erste Byte der Daten ist die eigenen TWI-Adresse
         sla_buf[sla_tail--] = temp - 1;  // TWI-Adresse durch Datensatzlnge berschreiben
         sla_buf[sla_tail] = LOGMODE;    // Frametyp

         int_buf  = sla_buf;              // Pointer auf das Array mit den Daten
         int_tail = sla_tail;             // Anfang und Ende bergeben
         int_head = sla_head;
         sla_tail = sla_head;             // damit die nchste Abfrage von TWI_SLA_Received False liefert

         UCSR0B = TX_ENABLE;              // Interrupt enablen
         UDR0 = STARTBYTE;                // Das erste Byte auf die Reise schicken
                                          // den Rest erledigt die ISR
      }
      TWI_SLA_Start_Transceiver();        // den Slave-Modus wieder starten
	}

}              // while 1
}              // main


// --- [ eof ] ----------------------------------------------------------------

