// --- [ rs232.c ] ------------------------------------------------------------
//       30.06.2013		tab = 3
//       zu pc_2_twi.c
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include "global.h"
#include "rs232.h"

//// ----------------------------------------------------------------------------
// gibt ein einzelnes Zeichen auf COM aus
void put_c(uint8_t Zeichen)
{
while(!(UCSR0A &(1<<UDRE0)));						// warten bis sendebereit
UDR0 = Zeichen;
}

// -----------------------------------------------------------------------------
void rs232_init(void)
{
UBRR0L = (uint8_t) TEILER;
UBRR0H = 0;
UCSR0B = TX_DISABLE;
UCSR0C = (0<<UMSEL01) | (1<<UCSZ01) | (1<<UCSZ00);
}

// ----------------------------------------------------------------------------
void put_msg(void)
// gibt den STARTBYTE  + Daten aus dem Array ber die Serielle Schnittstelle zurck
// Die Daten mssen im Array bereitgestellt sein,
// die Anzahl der Bytes == Head steht im 2. Byte
// Byte 0 = frametyp
{
   while(USART_BUSY);			// warten, falls noch eine bertragung luft

   int_buf  = msg;
   int_tail = 0;
   int_head = msg[1] + 2;     // Die Lnge steht in der msg + STARTBYTE + frametyp

   UCSR0B   = TX_ENABLE;
   UDR0     = STARTBYTE;
}


// ----------------------------------------------------------------------------
void put_err(uint8_t adr)
// gibt den STARTBYTE + Code fr Fehler ber die Serielle Schnittstelle zurck
{
   while(USART_BUSY);			// warten, falls noch eine bertragung luft

   msg[0] = TWI_ERR;
   msg[1] = 1;
   msg[2] = adr;

   int_buf  = msg;
   int_tail = 0;
   int_head = 3;

   UCSR0B = TX_ENABLE;
   UDR0   = STARTBYTE;
}


// ---------------------------------------------------------------------------
// wird aufgerufen, wenn ein Byte ber COM eingegangen ist
// hat nur die Aufgabe, den sleep-modus zu beenden.
// das Speichern des eingegangenen Bytes wird von main() bernommen

ISR(USART_RX_vect)
{
   rs_buf[rs_head++] = UDR0;
}

// ----------------------------------------------------------------------------
// Nachdem ein Byte verschickt worden ist, wird der TXC-Interrupt aufgerufen.
// Es wird geprft, ob weitere Daten im Ringpuffer sind.
// Wenn ja, dann wird das nchste Byte ausgegeben,
// wenn nicht, dann wird die bertragung beendet.
// Der Status von UCSR0B gibt zu erkennen, ob eine bertragung aktiv ist.
ISR (USART_TX_vect)
{
   if (!(int_tail == int_head))						// wenn rs_tail == rs_head, dann ist der Puffer leer
   {
      UDR0 = int_buf[int_tail++];					// Byte in den Sender schreiben
   }

   else													   // der Puffer ist leer, bertragung beenden
   {
      UCSR0B = TX_DISABLE;							   // TXC-Interrupt disablen
	}
}

// --- [ eof ] ----------------------------------------------------------------
